<?php
/**
 * Interface: for using Wordpress transient
 */
namespace EwTools\Interfaces\Storage;
/**
 * Interface WpTransientInterface
 *
 * @link    http://infomaker.se
 * @package EwTools\Interfaces\Storage
 * @since   EwTools\Interfaces\Storage\WpTransientInterface 0.1
 */
interface WpTransientInterface {
	
	/**
	 * Cache data
	 * @see set_transient()
	 *
	 * @param string $name
	 * @param mixed  $value
	 * @param int $expiration
	 *
	 * @since 0.1
	 * @return bool
	 */
	public static function set( $name, $value, $expiration = 0 );
	
	/**
	 * Cache data globally
	 * @see set_site_transient()
	 *
	 * @param string $name
	 * @param mixed  $value
	 * @param int $expiration
	 *
	 * @since 0.1
	 * @return bool
	 */
	public static function setGlobal( $name, $value, $expiration = 0 );
	
	/**
	 * Get cached data
	 * @see get_transient()
	 *
	 * @param string $name
	 * @param mixed  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public static function get( $name, $default = false );
	
	/**
	 * Get globally cached data
	 * @see get_site_transient()
	 *
	 * @param string $name
	 * @param mixed  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public static function getGlobal( $name, $default = false );
	
	/**
	 * Remove cached data
	 * @see delete_transient()
	 *
	 * @param string $name
	 *
	 * @since 0.1
	 * @return bool
	 */
	public static function remove( $name );
	
	/**
	 * Remove globally cached data
	 * @see delete_site_transient()
	 *
	 * @param string $name
	 *
	 * @since 0.1
	 * @return bool
	 */
	public static function removeGlobal( $name );
}