<?php
/**
 * Collection of helper functions for easy way of using support functions without having to "use" the class
 */

use EwTools\Support\Arr;
use EwTools\Support\Str;

if ( ! function_exists( 'array_add' ) ) {
	/**
	 * Add an element to an array using "dot" notation if it doesn't exist.
	 *
	 * @param  array  $array
	 * @param  string $key
	 * @param  mixed  $value
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_add( $array, $key, $value ) {
		return Arr::add( $array, $key, $value );
	}
}

if ( ! function_exists( 'array_collapse' ) ) {
	/**
	 * Collapse an array of arrays into a single array.
	 *
	 * @param  array|\ArrayAccess $array
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_collapse( $array ) {
		return Arr::collapse( $array );
	}
}

if ( ! function_exists( 'array_divide' ) ) {
	/**
	 * Divide an array into two arrays. One with keys and the other with values.
	 *
	 * @param  array $array
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_divide( $array ) {
		return Arr::divide( $array );
	}
}

if ( ! function_exists( 'array_dot' ) ) {
	/**
	 * Flatten a multi-dimensional associative array with dots.
	 *
	 * @param  array  $array
	 * @param  string $prepend
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_dot( $array, $prepend = '' ) {
		return Arr::dot( $array, $prepend );
	}
}

if ( ! function_exists( 'array_except' ) ) {
	/**
	 * Get all of the given array except for a specified array of items.
	 *
	 * @param  array        $array
	 * @param  array|string $keys
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_except( $array, $keys ) {
		return Arr::except( $array, $keys );
	}
}

if ( ! function_exists( 'array_first' ) ) {
	/**
	 * Return the first element in an array passing a given truth test.
	 *
	 * @param  array    $array
	 * @param  callable $callback
	 * @param  mixed    $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function array_first( $array, callable $callback, $default = null ) {
		return Arr::first( $array, $callback, $default );
	}
}

if ( ! function_exists( 'array_flatten' ) ) {
	/**
	 * Flatten a multi-dimensional array into a single level.
	 *
	 * @param  array $array
	 * @param  int   $depth
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_flatten( $array, $depth = INF ) {
		return Arr::flatten( $array, $depth );
	}
}

if ( ! function_exists( 'array_forget' ) ) {
	/**
	 * Remove one or many array items from a given array using "dot" notation.
	 *
	 * @param  array        $array
	 * @param  array|string $keys
	 *
	 * @since 0.1
	 * @return void
	 */
	function array_forget( &$array, $keys ) {
		Arr::forget( $array, $keys );
	}
}

if ( ! function_exists( 'array_get' ) ) {
	/**
	 * Get an item from an array using "dot" notation.
	 *
	 * @param  array  $array
	 * @param  string $key
	 * @param  mixed  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function array_get( $array, $key, $default = null ) {
		return Arr::get( $array, $key, $default );
	}
}

if ( ! function_exists( 'array_has' ) ) {
	/**
	 * Check if an item exists in an array using "dot" notation.
	 *
	 * @param  array  $array
	 * @param  string $key
	 *
	 * @since 0.1
	 * @return bool
	 */
	function array_has( $array, $key ) {
		return Arr::has( $array, $key );
	}
}

if ( ! function_exists( 'array_last' ) ) {
	/**
	 * Return the last element in an array passing a given truth test.
	 *
	 * @param  array    $array
	 * @param  callable $callback
	 * @param  mixed    $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function array_last( $array, $callback, $default = null ) {
		return Arr::last( $array, $callback, $default );
	}
}

if ( ! function_exists( 'array_only' ) ) {
	/**
	 * Get a subset of the items from the given array.
	 *
	 * @param  array        $array
	 * @param  array|string $keys
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_only( $array, $keys ) {
		return Arr::only( $array, $keys );
	}
}

if ( ! function_exists( 'array_pluck' ) ) {
	/**
	 * Pluck an array of values from an array.
	 *
	 * @param  array             $array
	 * @param  string|array      $value
	 * @param  string|array|null $key
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_pluck( $array, $value, $key = null ) {
		return Arr::pluck( $array, $value, $key );
	}
}

if ( ! function_exists( 'array_prepend' ) ) {
	/**
	 * Push an item onto the beginning of an array.
	 *
	 * @param  array $array
	 * @param  mixed $value
	 * @param  mixed $key
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_prepend( $array, $value, $key = null ) {
		return Arr::prepend( $array, $value, $key );
	}
}

if ( ! function_exists( 'array_pull' ) ) {
	/**
	 * Get a value from the array, and remove it.
	 *
	 * @param  array  $array
	 * @param  string $key
	 * @param  mixed  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function array_pull( &$array, $key, $default = null ) {
		return Arr::pull( $array, $key, $default );
	}
}

if ( ! function_exists( 'array_remove' ) ) {
	/**
	 * Remove a value from the array.
	 *
	 * @param  array  $array
	 * @param  string $value
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function array_remove( &$array, $value ) {
		if ( ( $key = array_search( $value, $array ) ) !== false ) {
			unset( $array[ $key ] );
		}
		
		return $array;
	}
}

if ( ! function_exists( 'array_set' ) ) {
	/**
	 * Set an array item to a given value using "dot" notation.
	 * If no key is given to the method, the entire array will be replaced.
	 *
	 * @param  array  $array
	 * @param  string $key
	 * @param  mixed  $value
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_set( &$array, $key, $value ) {
		return Arr::set( $array, $key, $value );
	}
}

if ( ! function_exists( 'array_sort' ) ) {
	/**
	 * Sort the array using the given callback.
	 *
	 * @param  array    $array
	 * @param  callable $callback
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_sort( $array, callable $callback ) {
		return Arr::sort( $array, $callback );
	}
}

if ( ! function_exists( 'array_sort_recursive' ) ) {
	/**
	 * Recursively sort an array by keys and values.
	 *
	 * @param  array $array
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_sort_recursive( $array ) {
		return Arr::sortRecursive( $array );
	}
}

if ( ! function_exists( 'array_where' ) ) {
	/**
	 * Filter the array using the given callback.
	 *
	 * @param  array    $array
	 * @param  callable $callback
	 *
	 * @since 0.1
	 * @return array
	 */
	function array_where( $array, callable $callback ) {
		return Arr::where( $array, $callback );
	}
}

if ( ! function_exists( 'dd' ) ) {
	/**
	 * Dump the passed variables and end the script.
	 *
	 * @param  mixed
	 *
	 * @since 0.1
	 * @return void
	 */
	function dd() {
		
		print_r( '<pre>' );
		var_dump( func_get_args() );
		print_r( '</pre>' );
		
		die( 1 );
	}
}

if ( ! function_exists( 'camel_case' ) ) {
	/**
	 * Convert a value to camel case.
	 *
	 * @param  string $value
	 *
	 * @since 0.1
	 * @return string
	 */
	function camel_case( $value ) {
		return Str::camel( $value );
	}
}

if ( ! function_exists( 'snake_case' ) ) {
	/**
	 * Convert a string to snake case.
	 *
	 * @param  string $value
	 * @param  string $delimiter
	 *
	 * @since 0.1
	 * @return string
	 */
	function snake_case( $value, $delimiter = '_' ) {
		return Str::snake( $value, $delimiter );
	}
}

if ( ! function_exists( 'ev_data_get' ) ) {
	/**
	 * Get an item from an array or object using "dot" notation.
	 *
	 * @param  mixed        $target
	 * @param  string|array $key
	 * @param  mixed        $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function ev_data_get( $target, $key, $default = null ) {
		if ( is_null( $key ) ) {
			return $target;
		}
		
		$key = is_array( $key ) ? $key : explode( '.', $key );
		
		while ( ( $segment = array_shift( $key ) ) !== null ) {
			if ( $segment === '*' ) {
				if ( ! is_array( $target ) && ! $target instanceof ArrayAccess ) {
					return $default;
				}
				
				$result = Arr::pluck( $target, $key );
				
				return in_array( '*', $key ) ? Arr::collapse( $result ) : $result;
			}
			
			if ( is_array( $target ) ) {
				if ( ! array_key_exists( $segment, $target ) ) {
					return ev_value( $default );
				}
				
				$target = $target[ $segment ];
			} elseif ( $target instanceof ArrayAccess ) {
				if ( ! isset( $target[ $segment ] ) ) {
					return ev_value( $default );
				}
				
				$target = $target[ $segment ];
			} elseif ( is_object( $target ) ) {
				if ( ! isset( $target->{$segment} ) ) {
					return ev_value( $default );
				}
				
				$target = $target->{$segment};
			} else {
				return ev_value( $default );
			}
		}
		
		return $target;
	}
}

if ( ! function_exists( 'ev_value' ) ) {
	/**
	 * Return the default value of the given value.
	 *
	 * @param  mixed $value
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function ev_value( $value ) {
		return $value instanceof Closure ? $value() : $value;
	}
}

if ( ! function_exists( 'ev_with' ) ) {
	/**
	 * Return the given object. Useful for chaining.
	 *
	 * @param  mixed $object
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function ev_with( $object ) {
		return $object;
	}
}

if ( ! function_exists( 'ev_default' ) ) {
	/**
	 * Return the given object. Useful for chaining.
	 *
	 * @param $value
	 * @param $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	function ev_default( $value, $default ) {
		return $value ? ev_value( $value ) : ev_value( $default );
	}
}