<?php namespace EwTools\Support;

/**
 * Utilities
 *
 * @link    http://infomaker.se
 * @package EwTools\Support
 * @since   EwTools\Support\Utilities 0.1
 */
class Utilities {
    
    /**
     * Function for fetching the correct js uri depending on if you are using the base- or child-theme's javascript
     *
     * @param string $file_name
     *
     * @since 0.1
     * @return string
     */
    public static function getCorrectJsUrl( $file_name = '' ) {
        return Str::finish( static::getStaticTemplateUri(), "/assets/js/{$file_name}" );
    }
    
    /**
     * Function to retrieve current host
     *
     * @since 0.1
     * @return mixed
     */
    public static function getCurrentHost() {
        return parse_url( site_url(), PHP_URL_HOST );
    }
    
    /**
     * Return path to plugin uri
     *
     * @param $file - filepath to the file requesting  (__FILE__)
     *
     * @since 0.1
     * @return mixed|string
     */
    public static function getStaticPluginUri( $file ) {
        $plugin_uri = plugin_dir_url( $file );
        $uri_index  = strpos( $plugin_uri, '/wp-content' );
        
        if( defined( CF_STATIC ) && ! empty( CF_STATIC ) ) {
            $plugin_uri = substr_replace( $plugin_uri, CF_STATIC, 0, $uri_index );
        }
        
        return $plugin_uri;
    }
    
    /**
     * Return base path of themes folder
     *
     * @since 0.1
     * @return string
     */
    public static function getStaticThemeBase() {
        $theme_uri = get_stylesheet_directory_uri();
        $uri_index = strrpos( $theme_uri, '/wp-content' );
        
        if( getenv( 'ENVIRONMENT' ) === 'prod' && CF_STATIC !== '' && CF_STATIC !== false ) {
            $theme_uri = rtrim( CF_STATIC, '/' ) . '/wp-content/themes/';
        } else {
            $theme_uri = substr( $theme_uri, 0, $uri_index ) . '/wp-content/themes/';
        }
        
        return $theme_uri;
    }
    
    /**
     * Return theme uri for static assets, if prod return with cloudfront.
     *
     * @since 0.1
     * @return string
     */
    public static function getStaticThemeUri() {
        
        $theme_uri = get_stylesheet_directory_uri();
        
        if( CF_STATIC !== '' && CF_STATIC !== false ) {
            $uri_index = strrpos( $theme_uri, '/wp-content' );
            $theme_uri = rtrim( CF_STATIC, '/' ) . substr( $theme_uri, $uri_index );
        }
        
        return $theme_uri;
    }
    
    /**
     * Return template uri for static assets, if prod return with cloudfront.
     *
     * @since 0.1
     * @return string
     */
    public static function getStaticTemplateUri() {
        
        $template_uri = get_template_directory_uri();
        
        if( CF_STATIC !== '' && CF_STATIC !== false ) {
            $uri_index    = strrpos( $template_uri, '/wp-content' );
            $template_uri = rtrim( CF_STATIC, '/' ) . substr( $template_uri, $uri_index );
        }
        
        return $template_uri;
    }
}
