<?php
/**
 * TextBlock widget.
 *
 * Plugin Name: Textblock
 * Description: Write arbitrary text, shortcodes or HTML
 * Version: 1.0
 * Author: Infomaker Scandinavia AB
 * Author URI: http://infomaker.se
 */
namespace EwWidgets;

use EwTools\Admin\Widget;
use EwTools\Handler\Shortcodes;

/**
 * TextBlock
 *
 * @link    http://infomaker.se
 * @package EwTools\Widgets
 * @since   EwTools\Widgets\TextBlock 0.1
 */
class TextBlock extends Widget {
    
    /**
     * Twig-template to use
     *
     * @var string
     */
    protected $template_path = '@tools/widgets/text-block';
    
    /**
     * Widget Fields
     * Contains fields to manage a headline and to blocks of text
     * with optional filter to wrap the text in paragraphs
     *
     * @var array
     */
    protected $fields = [
        'footer'        => '',
        'footer_filter' => false,
        'headline'      => '',
        'text'          => '',
        'text_filter'   => false
    ];
    
    /**
     * TextBlock constructor.
     *
     * @since 0.1
     */
    public function __construct() {
        $this->setWidgetData( __FILE__ );
        parent::__construct( 'ew-text-block' );
    }
    
    /**
     *
     *
     * @param $instance
     *
     * @since 0.1
     * @return string
     */
    protected function boardWidgetContent( $instance ) {
        $headline = $instance[ 'headline' ];
        $text = $this->getText( $instance[ 'text' ], $instance[ 'text_filter' ] );
        $footer = $this->getText( $instance[ 'footer' ], $instance[ 'footer_filter' ] );
        return "<p><strong>{$headline}</strong></p><div>{$text}</div><div>{$footer}</div>";
    }
    
    /**
     * Fires when rendering the widget form
     *
     * @param array $view_data
     *
     * @since 0.1
     * @return void
     */
    protected function edit( $view_data ) {
        $view_data[ 'title' ][ 'value' ] = $view_data[ 'headline' ][ 'value' ];
        $this->view( 'form', $view_data );
    }
    
    /**
     * Fires when rendering widget
     *
     * @param array $view_data
     *
     * @since 0.1
     * @return void
     */
    protected function show( $view_data ) {
        $view_data[ 'text' ]   = $this->getText( $view_data[ 'text' ], $view_data[ 'text_filter' ] );
        $view_data[ 'footer' ] = $this->getText( $view_data[ 'footer' ], $view_data[ 'footer_filter' ] );
        $this->view( 'widget', $view_data );
    }
    
    /**
     * Retrieve the text after running through shortcode and adding filter if set
     *
     * @param string $text
     * @param bool   $add_paragraphs
     *
     * @since 0.1
     * @return string
     */
    protected function getText( $text, $add_paragraphs = false ) {
        return Shortcodes::generate( $add_paragraphs ? wpautop( $text ) : $text );
    }
}